use binrw::helpers::until_eof;
use binrw::prelude::*;

#[derive(Debug, BinRead)]
enum BinaryMessage {
    #[br(magic = 0u8)]
    Push(PushBinaryMessage),
    #[br(magic = 1u8)]
    Reply(ReplyBinaryMessage),
    #[br(magic = 2u8)]
    Broadcast(BroadcastBinaryMessage),
}

#[derive(Debug, BinRead)]
struct PushBinaryMessage {
    ref_size: u8,
    join_ref_size: u8,
    _topic_size: u8,
    _event_size: u8,
    #[br(count = ref_size)]
    r#ref: Vec<u8>,
    #[br(count = join_ref_size)]
    join_ref: Vec<u8>,
    #[br(count = _topic_size)]
    topic: Vec<u8>,
    #[br(count = _event_size)]
    event: Vec<u8>,
    #[br(parse_with = until_eof)]
    payload: Vec<u8>,
}

#[derive(Debug, BinRead)]
struct ReplyBinaryMessage {
    join_ref_size: u8,
    ref_size: u8,
    _topic_size: u8,
    _status_size: u8,
    #[br(count = join_ref_size)]
    join_ref: Vec<u8>,
    #[br(count = ref_size)]
    r#ref: Vec<u8>,
    #[br(count = _topic_size)]
    topic: Vec<u8>,
    #[br(count = _status_size)]
    status: Vec<u8>,
    #[br(parse_with = until_eof)]
    payload: Vec<u8>,
}

#[derive(Debug, BinRead)]
struct BroadcastBinaryMessage {
    _topic_size: u8,
    _event_size: u8,
    #[br(count = _topic_size)]
    topic: Vec<u8>,
    #[br(count = _event_size)]
    event: Vec<u8>,
    #[br(parse_with = until_eof)]
    payload: Vec<u8>,
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn parses_binary() {
        // broadcast!("agents:1", "subject", {:binary, "hello"})
        let raw_msg = vec![
            2, 8, 7, 97, 103, 101, 110, 116, 115, 58, 49, 115, 117, 98, 106, 101, 99, 116, 104,
            101, 108, 108, 111,
        ];

        parse_binary_message(&raw_msg[..]).unwrap();
    }

    #[test]
    fn parses_json() {
        // broadcast!("agents:1", "subject", "hello")
        let raw_msg = r#"[null, "0", "agents:1", "subject", "hello"]"#;
        parse_json_message(raw_msg).unwrap();
    }
}
